@@ -1,4 +1,4 @@
-# Copyright 2020 NREL
+# Copyright 2021 NREL
 
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not
 # use this file except in compliance with the License. You may obtain a copy of
@@ -18,8 +18,7 @@ import numpy as np
 import pandas as pd
 from scipy.optimize import minimize
 
-from floris.tools.optimization.scipy.yaw_wind_rose import YawOptimizationWindRose
-
+from .yaw_wind_rose import YawOptimizationWindRose
 from ....logging_manager import LoggerBase
 
 
@@ -44,6 +43,7 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
         maximum_yaw_angle=25.0,
         minimum_ws=3.0,
         maximum_ws=25.0,
+        yaw_angles_baseline=None,
         x0=None,
         bnds=None,
         opt_method="SLSQP",
@@ -51,6 +51,8 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
         include_unc=False,
         unc_pmfs=None,
         unc_options=None,
+        turbine_weights=None,
+        exclude_downstream_turbines=False,
     ):
         """
         Instantiate YawOptimizationWindRoseParallel object with a
@@ -68,22 +70,49 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                 specified, the current TI value in the Floris object will be
                 used for all optimizations. Defaults to None.
             minimum_yaw_angle (float, optional): Minimum constraint on yaw
-                angle (deg). Defaults to 0.0.
+                angle (deg). This value will be ignored if bnds is also
+                specified. Defaults to 0.0.
             maximum_yaw_angle (float, optional): Maximum constraint on yaw
-                angle (deg). Defaults to 25.0.
+                angle (deg). This value will be ignored if bnds is also
+                specified. Defaults to 25.0.
             minimum_ws (float, optional): Minimum wind speed at which
                 optimization is performed (m/s). Assumes zero power generated
                 below this value. Defaults to 3.
             maximum_ws (float, optional): Maximum wind speed at which
                 optimization is performed (m/s). Assumes optimal yaw offsets
                 are zero above this wind speed. Defaults to 25.
-            x0 (iterable, optional): The initial yaw conditions (deg). If none
-                are specified, they are set to the current yaw angles for all
-                turbines. Defaults to None.
-            bnds (iterable, optional): Bounds for the yaw angles (tuples of
-                min, max values for each turbine (deg)). If none are specified,
-                they are set to (minimum_yaw_angle, maximum_yaw_angle) for each
-                turbine. Defaults to None.
+            yaw_angles_baseline (iterable, optional): The baseline yaw
+                angles used to calculate the initial and baseline power
+                production in the wind farm and used to normalize the cost
+                function. If none are specified, this variable is set equal
+                to the current yaw angles in floris. Note that this variable
+                need not meet the yaw constraints specified in self.bnds,
+                yet a warning is raised if it does to inform the user.
+                Defaults to None.
+            x0 (iterable, optional): The initial guess for the optimization
+                problem. These values must meet the constraints specified
+                in self.bnds. Note that, if exclude_downstream_turbines=True,
+                the initial guess for any downstream turbines are ignored
+                since they are not part of the optimization. Instead, the yaw
+                angles for those turbines are 0.0 if that meets the lower and
+                upper bound, or otherwise as close to 0.0 as feasible. If no
+                values for x0 are specified, x0 is set to be equal to zeros
+                wherever feasible (w.r.t. the bounds), and equal to the
+                average of its lower and upper bound for all non-downstream
+                turbines otherwise. Defaults to None.
+            bnds (iterable, optional): Bounds for the yaw angles, as tuples of
+                min, max values for each turbine (deg). One can fix the yaw
+                angle of certain turbines to a predefined value by setting that
+                turbine's lower bound equal to its upper bound (i.e., an
+                equality constraint), as: bnds[ti] = (x, x), where x is the
+                fixed yaw angle assigned to the turbine. This works for both
+                zero and nonzero yaw angles. Moreover, if 
+                exclude_downstream_turbines=True, the yaw angles for all
+                downstream turbines will be 0.0 or a feasible value closest to
+                0.0. If none are specified, the bounds are set to
+                (minimum_yaw_angle, maximum_yaw_angle) for each turbine. Note
+                that, if bnds is not none, its values overwrite any value given
+                in minimum_yaw_angle and maximum_yaw_angle. Defaults to None.
             opt_method (str, optional): The optimization method used by
                 scipy.optimize.minize. Defaults to 'SLSQP'.
             opt_options (dictionary, optional): Optimization options used by
@@ -92,15 +121,15 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                 'eps': 0.01}. Defaults to None.
             include_unc (bool, optional): Determines whether wind direction or
                 yaw uncertainty are included. If True, uncertainty in wind
-                direction and/or yaw position is included when determining wind
-                farm power. Uncertainty is included by computing the mean wind
-                farm power for a distribution of wind direction and yaw
-                position deviations from the intended wind direction and yaw
-                angles. Defaults to False.
+                direction and/or yaw position is included when determining
+                wind farm power. Uncertainty is included by computing the
+                mean wind farm power for a distribution of wind direction
+                and yaw position deviations from the intended wind direction
+                and yaw angles. Defaults to False.
             unc_pmfs (dictionary, optional): A dictionary containing
-                probability mass functions describing the distribution of wind
-                direction and yaw position deviations when wind direction and
-                or yaw position uncertainty is included in the power
+                probability mass functions describing the distribution of
+                wind direction and yaw position deviations when wind direction
+                and/or yaw position uncertainty is included in the power
                 calculations. Contains the following key-value pairs:
 
                 -   **wd_unc** (*np.array*): The wind direction
@@ -135,23 +164,42 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                 If none are specified, default values of
                 {'std_wd': 4.95, 'std_yaw': 1.75, 'pmf_res': 1.0,
                 'pdf_cutoff': 0.995} are used. Defaults to None.
+            turbine_weights (iterable, optional): weighing terms that allow
+                the user to emphasize power gains at particular turbines or
+                completely ignore power gains from other turbines. The array
+                of turbine powers from floris is multiplied with this array
+                in the calculation of the objective function. If None, this
+                is an array with all values 1.0 and length equal to the
+                number of turbines. Defaults to None.
+            exclude_downstream_turbines (bool, optional): If True,
+                automatically finds and excludes turbines that are most
+                downstream from the optimization problem. This significantly
+                reduces computation time at no loss in performance. The yaw
+                angles of these downstream turbines are fixed to 0.0 deg if
+                the yaw bounds specified in self.bnds allow that, or otherwise
+                are fixed to the lower or upper yaw bound, whichever is closer
+                to 0.0. Defaults to False.
         """
         super().__init__(
             fi,
             wd,
             ws,
-            ti=None,
-            minimum_yaw_angle=0.0,
-            maximum_yaw_angle=25.0,
-            minimum_ws=3.0,
-            maximum_ws=25.0,
-            x0=None,
-            bnds=None,
-            opt_method="SLSQP",
-            opt_options=None,
-            include_unc=False,
-            unc_pmfs=None,
-            unc_options=None,
+            ti=ti,
+            minimum_yaw_angle=minimum_yaw_angle,
+            maximum_yaw_angle=maximum_yaw_angle,
+            minimum_ws=minimum_ws,
+            maximum_ws=maximum_ws,
+            yaw_angles_baseline=yaw_angles_baseline,
+            x0=x0,
+            bnds=bnds,
+            opt_method=opt_method,
+            opt_options=opt_options,
+            include_unc=include_unc,
+            unc_pmfs=unc_pmfs,
+            unc_options=unc_options,
+            turbine_weights=turbine_weights,
+            calc_init_power=False,
+            exclude_downstream_turbines=exclude_downstream_turbines,
         )
 
     # Private methods
@@ -218,15 +266,14 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                     wind_direction=wd, wind_speed=ws, turbulence_intensity=ti
                 )
             # calculate baseline power
-            self.fi.calculate_wake(yaw_angles=0.0)
+            self.fi.calculate_wake(yaw_angles=self.yaw_angles_baseline)
             power_base = self.fi.get_turbine_power(
                 include_unc=self.include_unc,
                 unc_pmfs=self.unc_pmfs,
                 unc_options=self.unc_options,
             )
-
             # calculate power for no wake case
-            self.fi.calculate_wake(no_wake=True)
+            self.fi.calculate_wake(yaw_angles=self.yaw_angles_baseline, no_wake=True)
             power_no_wake = self.fi.get_turbine_power(
                 include_unc=self.include_unc,
                 unc_pmfs=self.unc_pmfs,
@@ -237,6 +284,10 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
             power_base = self.nturbs * [0.0]
             power_no_wake = self.nturbs * [0.0]
 
+        # Add turbine weighing terms
+        power_base = np.multiply(self.turbine_weights, power_base)
+        power_no_wake = np.multiply(self.turbine_weights, power_no_wake)
+
         # add variables to dataframe
         if ti is None:
             df_base = pd.DataFrame(
@@ -264,7 +315,7 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
 
         return df_base
 
-    def _optimize_one_case(self, ws, wd, ti=None):
+    def _optimize_one_case(self, ws, wd, initial_farm_power, ti=None):
         """
         For a single (wind speed, direction, ti (optional)) combination, finds
         the power resulting from optimal wake steering.
@@ -323,6 +374,7 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                     wind_direction=wd, wind_speed=ws, turbulence_intensity=ti
                 )
 
+            self.initial_farm_power = initial_farm_power
             opt_yaw_angles = self._optimize()
 
             if np.sum(opt_yaw_angles) == 0:
@@ -349,8 +401,8 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                 self.fi.reinitialize_flow_field(
                     wind_direction=wd, wind_speed=ws, turbulence_intensity=ti
                 )
-            self.fi.calculate_wake(yaw_angles=0.0)
-            opt_yaw_angles = self.nturbs * [0.0]
+            opt_yaw_angles = np.array(self.yaw_angles_template, copy=True)
+            self.fi.calculate_wake(yaw_angles=opt_yaw_angles)
             power_opt = self.fi.get_turbine_power(
                 include_unc=self.include_unc,
                 unc_pmfs=self.unc_pmfs,
@@ -361,9 +413,12 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                 "No change in controls suggested for this inflow \
                     condition..."
             )
-            opt_yaw_angles = self.nturbs * [0.0]
+            opt_yaw_angles = np.array(self.yaw_angles_template, copy=True)
             power_opt = self.nturbs * [0.0]
 
+        # Add turbine weighing terms
+        power_opt = np.multiply(self.turbine_weights, power_opt)
+
         # add variables to dataframe
         if ti is None:
             df_opt = pd.DataFrame(
@@ -460,6 +515,7 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
 
         df_base.reset_index(drop=True, inplace=True)
 
+        self.df_base = df_base
         return df_base
 
     def optimize(self):
@@ -505,7 +561,7 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
         print("=====================================================")
         print("Optimizing wake redirection control in parallel...")
         print("Number of wind conditions to optimize = ", len(self.wd))
-        print("Number of yaw angles to optimize = ", len(self.x0))
+        print("Number of yaw angles to optimize = ", len(self.turbs_to_opt))
         print("=====================================================")
 
         df_opt = pd.DataFrame()
@@ -513,7 +569,10 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
         with MPIPoolExecutor() as executor:
             if self.ti is None:
                 for df_opt_one in executor.map(
-                    self._optimize_one_case, self.ws.values, self.wd.values
+                    self._optimize_one_case,
+                    self.ws.values,
+                    self.wd.values,
+                    self.df_base.power_baseline.values,
                 ):
 
                     # add variables to dataframe
@@ -523,6 +582,7 @@ class YawOptimizationWindRoseParallel(YawOptimizationWindRose, LoggerBase):
                     self._optimize_one_case,
                     self.ws.values,
                     self.wd.values,
+                    self.df_base.power_baseline.values,
                     self.ti.values,
                 ):
 
